--[[
	Keyboard NKeyboard 1.0v (With help from xerpi)
	Create by NEKERAFA (nekerafa@gmail.com) on fri 12 Aug 2011, 14:18:56 (CEST)
	License by Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Spain
	http://creativecommons.org/licenses/by-nc-sa/3.0/es/legalcode.es
	
	# nkeyboard-lib.lua
	#
	# This library loads functions for use NKeyboard
]]--

-- ## Tables ## --
-- Main Table --
nkeyboard = {}

-- Character Map --
nkeyboard.map = {
	upp = { -- Capital Characters --
		{"Q", "W", "E", "R", "T", "Y", "U", "I", "O", "1", "2", "3"},
		{"P", "A", "S", "D", "F", "G", "H", "J", "K", "4", "5", "6"},
		{"L", "Z", "X", "C", "V", "B", "N", "M", "7", "8", "9", "0"}
	},
	lower = { -- Lowercase --
		{"q", "w", "e", "r", "t", "y", "u", "i", "o", ".", "-", "_"},
		{"p", "a", "s", "d", "f", "g", "h", "j", "k", ",", "(", ")"},
		{"l", "z", "x", "c", "v", "b", "n", "m", ":", ";", "[", "]"}
	},
	sym = { -- Symbols --
		{"\225", "\233", "\237", "\243", "\250", "\193", "\201", "\205", "\211", "\218", "¡", "!"},
		{"\224", "\234", "\236", "\244", "\249", "\192", "\200", "\204", "\210", "\217", "\191", "?"},
		{"\231", "\199", "\241", "\209", "{", "}", "$", "%", "/", "*", "+", "\\"}
	}
}

-- Variables --
nkeyboard.vars = {
	pot = 0, x = 1, y = 1, xtext = 347,
	upp = image.load("img/upper.png"), lower = image.load("img/lower.png"), sym = image.load("img/symbols.png"),
	select = {
		img = image.load("img/select.png"), upp = image.load("img/supper.png"), lower = image.load("img/slower.png"), sym = image.load("img/ssymbol.png")
	},
}

-- ## Functions ## --
--[[
	text = nkeyboard.osk( -- New keyboard --
			text: initial text
			comment: text description
			* maxsize: limit of text
			* select: initial selection of letters (0 Capital letters, 1 Lowercase letters, 2 Symbols and specials character)
			(*) Optional
		)
]]--
function nkeyboard.osk(text, comment, maxsize,  select)
	local textreturn = text -- Return default text  --

	if select then -- If exist selection, configure  keyboard
		if select == 0 then nkeyboard.vars.selection = "upp"
		elseif select == 1 then nkeyboard.vars.selection = "lower"
		elseif select == 2 then nkeyboard.vars.selection = "sym" end
	else nkeyboard.vars.selection = "upp" end

	-- I create some variables --
	nkeyboard.vars.text = text -- Save default text --
	nkeyboard.vars.bg = screen.toimage() -- Save a screenshot --
	nkeyboard.vars.pos = #text -- Position the pointer at text's end --
	pcall(dofile, "search.lua")

	-- Main while --
	while true do
		controls.read()
		nkeyboard.vars.bg:blend(0,0,50)

		-- Move selected characters --
		if controls.press("up") and nkeyboard.vars.y > 1 then nkeyboard.vars.y = nkeyboard.vars.y-1
		elseif controls.press("down") and nkeyboard.vars.y < 3 then nkeyboard.vars.y = nkeyboard.vars.y+1 end
		if controls.press("left") and nkeyboard.vars.x > 1 then nkeyboard.vars.x = nkeyboard.vars.x-1
		elseif controls.press("right") and nkeyboard.vars.x < 12 then nkeyboard.vars.x = nkeyboard.vars.x+1 end

		-- Insert character --
		if controls.press("cross") and nkeyboard.vars.pos < #nkeyboard.vars.text then
			nkeyboard.vars.text = string.sub(nkeyboard.vars.text, 1, nkeyboard.vars.pos)..nkeyboard.map[nkeyboard.vars.selection][nkeyboard.vars.y][nkeyboard.vars.x]..string.sub(nkeyboard.vars.text, nkeyboard.vars.pos+1)
			nkeyboard.vars.pos = nkeyboard.vars.pos+1
		elseif controls.press("cross") and nkeyboard.vars.pos == #nkeyboard.vars.text then
			nkeyboard.vars.text = nkeyboard.vars.text..nkeyboard.map[nkeyboard.vars.selection][nkeyboard.vars.y][nkeyboard.vars.x]
			nkeyboard.vars.pos = nkeyboard.vars.pos+1
		end

		-- Detele character --
		if controls.press("square") and nkeyboard.vars.pos < #nkeyboard.vars.text then
			nkeyboard.vars.text = string.sub(nkeyboard.vars.text, 1, nkeyboard.vars.pos-1)..string.sub(nkeyboard.vars.text, nkeyboard.vars.pos+1)
			nkeyboard.vars.pos = nkeyboard.vars.pos-1
		elseif controls.press("square") and nkeyboard.vars.pos == #nkeyboard.vars.text then
			nkeyboard.vars.text = string.sub(nkeyboard.vars.text, 1, #nkeyboard.vars.text-1); nkeyboard.vars.pos = nkeyboard.vars.pos-1
		end

		-- Space --
		if controls.press("triangle") and nkeyboard.vars.pos < #nkeyboard.vars.text then
			nkeyboard.vars.text = string.sub(nkeyboard.vars.text, 1, nkeyboard.vars.pos).." "..string.sub(nkeyboard.vars.text, nkeyboard.vars.pos+1); nkeyboard.vars.pos = nkeyboard.vars.pos+1
		elseif controls.press("triangle") and nkeyboard.vars.pos == #nkeyboard.vars.text then
			nkeyboard.vars.text = string.sub(nkeyboard.vars.text, 1).." "; nkeyboard.vars.pos = nkeyboard.vars.pos+1
		end

		nkeyboard.vars.text = string.sub(nkeyboard.vars.text, 1, maxsize or 38)
		nkeyboard.vars.pos = math.min(math.max(0, nkeyboard.vars.pos), #nkeyboard.vars.text)

		-- Exit keyboard --
		if controls.press("circle") then
			cancel = os.message("Do you want to cancel text entry?", 1)
			if cancel == 1 then break end
		end

		-- Change selected characters --
		if controls.press("select") and nkeyboard.vars.selection == "upp" then nkeyboard.vars.selection = "lower"
		elseif controls.press("select") and nkeyboard.vars.selection == "lower" then nkeyboard.vars.selection = "sym"
		elseif controls.press("select") and nkeyboard.vars.selection == "sym" then nkeyboard.vars.selection = "upp" end

		-- Save keyboard text --
		if controls.press("start") and nkeyboard.vars.text ~= "" then
			textreturn = nkeyboard.vars.text
			if not search then
				updatesearch = io.open("search.lua", "w+")
				updatesearch:write("search = {\""..nkeyboard.vars.text.."\"}")
				updatesearch:close()
				updatesearch = nil
			else
				updatesearch = io.open("search.lua", "r+")
				oldsearch = updatesearch:read("*a")
				updatesearch:close()
				if not string.find(oldsearch, nkeyboard.vars.text) then
					updatesearch = io.open("search.lua", "w+")
					updatesearch:write(string.sub(oldsearch, 1, string.find(oldsearch, "}")-1)..", \""..nkeyboard.vars.text.."\"}")
					updatesearch:close()
				end
				oldsearch = nil; updatesearch = nil
			end
			break
		end

		-- Show search --
		if search and nkeyboard.vars.text ~= "" then
			for n = 1, #search do
				if string.sub(search[n], 1, #nkeyboard.vars.text) == nkeyboard.vars.text then newsearch = search[n]; break end
			end
			if newsearch and #nkeyboard.vars.text ~= #newsearch then screen.print(8, 8, newsearch, 0.8, color.new(240, 240, 240, 160), color.new(0,0,0,80)) end
		end

		-- Move pointer --
		if controls.press("l") and nkeyboard.vars.pos > 0 then nkeyboard.vars.pos = nkeyboard.vars.pos-1
		elseif controls.press("r") and newsearch then if nkeyboard.vars.pos < #newsearch then nkeyboard.vars.text = newsearch; nkeyboard.vars.pos = #newsearch end
		elseif controls.press("r") and not newsearch then if nkeyboard.vars.pos < #nkeyboard.vars.text then nkeyboard.vars.pos = nkeyboard.vars.pos+1 end
		end

		-- For restart pointer --
		if controls.cross() then nkeyboard.vars.pot = 0; nkeyboard.vars.status = true
		elseif controls.square() or controls.triangle() or controls.r() or controls.l() then nkeyboard.vars.pot = 0 end

		-- Show text --
		screen.print(8, 8, nkeyboard.vars.text, 0.8, color.new(240, 240, 240, 240), color.new(0,0,0,80))
		-- Show pointer --
		if nkeyboard.vars.pot < 1 then screen.print(8+screen.textwidth(string.sub(nkeyboard.vars.text, 0, nkeyboard.vars.pos), 0.8), 8,"|", 0.8, color.new(240, 240, 240, 240), color.new(0,0,0,80))
		elseif nkeyboard.vars.pot > 2 then nkeyboard.vars.pot = 0 end

		-- Show keyboard --
		nkeyboard.vars[nkeyboard.vars.selection]:blit(0, 198)
		-- Show selecction --
		if nkeyboard.vars.status then nkeyboard.vars.select.img:blend((121+((nkeyboard.vars.x-1)*20)), (214+((nkeyboard.vars.y-1)*19)), 160)
		else nkeyboard.vars.select.img:blit((121+((nkeyboard.vars.x-1)*20)), (214+((nkeyboard.vars.y-1)*19))) end

		-- Show selected characters icon --
		nkeyboard.vars.select[nkeyboard.vars.selection]:blit(321, 199)
		nkeyboard.vars.xtext = screen.print(nkeyboard.vars.xtext, 200, comment, 0.6, color.new(240, 240, 240, 240), color.new(0,0,0,80), "scroll_through", 130)

		nkeyboard.vars.pot = nkeyboard.vars.pot+0.02
		nkeyboard.vars.status = false
		newsearch = nil
		screen.flip()
	end

	cancel = nil
	nkeyboard.vars.bg:free()
	nkeyboard.vars.bg = nil
	nkeyboard.vars.status = nil
	nkeyboard.vars.text = nil
	nkeyboard.vars.pos = nil
	nkeyboard.vars.pot = 0
	nkeyboard.vars.x = 1
	nkeyboard.vars.y = 1
	nkeyboard.vars.xtext = 347
	return textreturn
end
