--[[
	Google Maps (PSP version)
	Create by XERPI and NEKERAFA
]]
dofile("nkeyboard.lua")
logo = image.load("img/logo.png")
logo:center()
scenery = image.load("img/scenery.png")
scenery:center()

xml = {} -- Tabla para leer archivos xml
function xml.readtag(text, tag) -- xml.readtag(texto, nombre del tag): Devuelve lo que contiene el tag xml
	return string.sub(text, string.find(text, "<"..tag..">")+2+#tag, string.find(text, "</"..tag..">")-1) or ""
end

function xml.read(file, tag, head) -- , isnumber) -- xml.read(texto, tag, tag raiz): Devuelve lo que contiene el tag dentro de un tag xml
	local xmlfile = io.open(file, "r+"); local xmltext = xmlfile:read("*a"); xmlfile:close(); xmlfile = nil
	if head then return xml.readtag(xml.readtag(xmltext, head),tag)
	else return xml.readtag(xmltext, tag) end
end

map = { -- Tabla de variables de los mapas
	search = "Spain", -- Ultima busqueda
	-- socket = socket.connect("http://maps.google.com/",80),
	location = {}, -- Ubicación: lat (Latitud) y lng (Longitud)
	zoomdate = 4,
	maptype = {"roadmap", "satellite", "terrain", "hybrid", current = 4},
	mapspanishtype = {"Relevo", "Satélite", "Terreno", "Híbrido"},
	showinfo = true
}

zoom = {
200,
180,
150,
100,
50,
10,
5,
2,
1,
0.64,
0.32,
0.16,
0.08,
0.04,
0.02,
0.01,
0.005,
0.002,
0.001,
0.0005,
}

	-- ## Funciones generales ## --
	map.update =  {} -- Funciones de actualización del mapa
	-- Función para actualizar la imagen del mapa via http
	function map.update.http()
		http.get("http://maps.google.com/maps/api/staticmap?center="..map.location.lat..","..map.location.lng.."&format=gif&maptype="..map.maptype[map.maptype.current].."&zoom="..map.zoomdate.."&size=480x300&sensor=true","map.gif")
		if map.img then map.img:free() end
		map.img = image.load("map.gif")
		map.img:center()
		files.remove("map.gif")
	end

	-- Función para actualizar la imagen del mapa via sockets
	function map.update.socket()
		-- ??? --
	end

	-- Función para actualizar el archivo xml
	function map.update.xml()
		http.get("http://maps.google.com/maps/api/geocode/xml?address="..string.gsub(string.lower(map.search), "%s", "+").."&sensor=false","coord.txt")
		map.location.lat = tonumber(xml.read("coord.txt", "lat","location"))
		map.location.lng = tonumber(xml.read("coord.txt", "lng","location"))
		files.remove("coord.txt")
	end

	-- Función para cambiar los valores de las tablas
	function map.change(tab, dif)
		tab.current = tab.current + dif
		if tab.current < 1 then tab.current = #tab end
		if tab.current > #tab then tab.current = 1 end
	end

	-- Función para hacer zoom en el mapa
	function map.zoom()
		if controls.press("r") then
			map.zoomdate = math.min(20,map.zoomdate+2)
			map.update.http()
		elseif controls.press("l") then 
			map.zoomdate = math.max(1,map.zoomdate-2)
			map.update.http()
		end
	end

	-- Función para mover el mapa
		function map.move()
			if controls.press("up") then
				map.location.lat = map.location.lat+zoom[map.zoomdate]/4
				map.update.http()
			elseif controls.press("right") then 
				map.location.lng = map.location.lng+zoom[map.zoomdate]/4
				map.update.http()
			elseif controls.press("left") then 
				map.location.lng = map.location.lng-zoom[map.zoomdate]/4
				map.update.http()
			elseif controls.press("down") then 
				map.location.lat = map.location.lat-zoom[map.zoomdate]/4
				map.update.http()
			end
		end
		
	-- Función para buscar
		function map.searchplace()
			if controls.press("cross") then
				map.search = nkeyboard.osk(map.search or "", "City, State", 39)
				map.zoomdate = 4
				map.update.xml()
				map.update.http()
			end
		end
		
	-- Función para hacer captura de pantalla
		function map.makescreenshot()
			local psp = "ms0:/"
			if files.exists("ef0:/") then psp = "ef0:/" end
			if not files.exists(psp.."PICTURE") then files.mkdir(psp.."PICTURE") end
			if not files.exists(psp.."PICTURE/PSPMaps") then files.mkdir(psp.."PICTURE/PSPMaps") end
			local count = 1
			while true do	
				if not files.exists(psp.."PICTURE/PSPMaps/"..map.search..count..".png") then break end
				count = count +1			
			end
			local img = image.create(480,272)
			img:blit(0,0,map.img)
			img:save(psp.."PICTURE/PSPMaps/"..map.search..count..".png")
			img:free()
		end
		
-- ## Inicio ## --
for n = 0, 255, 5 do
	scenery:blend(240, 136, n)
	screen.flip()
end
os.sleep(2)
for n = 255, 0, -5 do
	scenery:blend(240, 136, n)
	screen.flip()
end

for n = 0, 100 do
	logo:factorscale(math.max(60, 100-n))
	screen.bilinear(1)
	logo:blend(240, 136, math.ceil(n*255/100))
	screen.bilinear(0)
	screen.flip()
end
for alpha = 0, 255, 5 do
	screen.bilinear(1)
	logo:blit(240, 136)
	screen.bilinear(0)
	screen.print(320, 260, "By XERPI and NEKERAFA", 0.6, color.new(255, 255, 255, alpha), color.new(0,0,0,80))
	screen.flip()
end
os.sleep(1)

screen.bilinear(1)
logo:blit(240, 136)
screen.bilinear(0)
screen.print(320, 260, "By XERPI and NEKERAFA", 0.6, color.new(255, 255, 255), color.new(0,0,0,80))
draw.fillrect(0, 0, 480, 272, color.new(200, 220, 255, 140))
screen.flip()
wlan.init() -- Iniciamos el wifi
map.update.xml() -- Descargamos las coordenadas
map.update.http() -- Descargamos el mapa

for n = 100, 0,-1 do
	logo:factorscale(math.max(60, 100-n))
	map.img:blit(240,150)
	screen.bilinear(1)
	logo:blend(240, 136, math.ceil(n*255/100))
	screen.bilinear(0)
	screen.flip()
	
end
logo:factorscale(60)

while true do
	controls.read()
	screen.bilinear(1)
	logo:blit(240, 136)
	screen.bilinear(0)
	map.img:blit(240, 150)
	if map.showinfo then
		screen.print(5, 255, "Búsqueda: "..map.search,0.7,color.new(255,0,0),color.new(0,0,0))
		screen.print(5, 5, "Zoom: "..map.zoomdate,0.7,color.new(0,255,0),color.new(0,0,0))
		screen.print(20 + screen.textwidth("Zoom: "..map.zoomdate,0.7), 5, "Tipo de mapa: "..map.mapspanishtype[map.maptype.current],0.7,color.new(0,0,255),color.new(0,0,0))
		screen.print(35 + screen.textwidth("Zoom: "..map.zoomdate.."Tipo de mapa: "..map.mapspanishtype[map.maptype.current],0.7), 5, "Lat/Lng: "..map.location.lat.."/"..map.location.lng,0.7,color.new(255,255,0),color.new(0,0,0))
	end
	if controls.press("start") then map.showinfo = not map.showinfo end
	--screen.print(5, 5, map.search.."   zoom: "..map.zoomdate.."   maptype: "..map.maptype[map.maptype.current], 0.7, color.new(255, 255, 255), color.new(0,0,0,100))
	--screen.print(5,25, map.location.lat, 0.8, color.new(255, 255, 255), color.new(0,0,0,100))
	--screen.print(5,45, map.location.lng, 0.8, color.new(255, 255, 255), color.new(0,0,0,100))
	if controls.press("triangle") then map.change(map.maptype, 1); map.update.http() end
	map.searchplace()
	map.zoom()
	map.move()
	screen.flip()
	if controls.press("select") then map.makescreenshot() end
end
